@@ -1,12 +1,13 @@
 pub mod chi_squared;
+pub mod params;
 
 pub use chi_squared::*;
+pub use params::*;
 
 use crate::DistributionError;
 use crate::{DependentJoint, Distribution, IndependentJoint, RandomVariable};
 use rand::prelude::*;
 use rand_distr::Gamma as RandGamma;
-use special::Gamma as GammaFunc;
 use std::{ops::BitAnd, ops::Mul};
 
 /// Gamma distribution
@@ -22,19 +23,21 @@ pub enum GammaError {
 }
 
 impl Distribution for Gamma {
-    type T = f64;
-    type U = GammaParams;
+    type Value = f64;
+    type Condition = GammaParams;
 
-    fn p(&self, x: &Self::T, theta: &Self::U) -> Result<f64, DistributionError> {
+    fn fk(&self, x: &Self::Value, theta: &Self::Condition) -> Result<f64, DistributionError> {
         let shape = theta.shape();
         let scale = theta.scale();
 
-        Ok((1.0 / GammaFunc::gamma(shape) * scale.powf(shape))
-            * x.powf(shape - 1.0)
-            * (-x / scale).exp())
+        Ok(x.powf(shape - 1.0) * (-x / scale).exp())
     }
 
-    fn sample(&self, theta: &Self::U, rng: &mut StdRng) -> Result<Self::T, DistributionError> {
+    fn sample(
+        &self,
+        theta: &Self::Condition,
+        rng: &mut dyn RngCore,
+    ) -> Result<Self::Value, DistributionError> {
         let shape = theta.shape();
         let scale = theta.scale();
 
@@ -47,40 +50,9 @@ impl Distribution for Gamma {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct GammaParams {
-    shape: f64,
-    scale: f64,
-}
-
-impl GammaParams {
-    pub fn new(shape: f64, scale: f64) -> Result<Self, DistributionError> {
-        if shape <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                GammaError::ShapeMustBePositive.into(),
-            ));
-        }
-        if scale <= 0.0 {
-            return Err(DistributionError::InvalidParameters(
-                GammaError::ScaleMustBePositive.into(),
-            ));
-        }
-
-        Ok(Self { shape, scale })
-    }
-
-    pub fn shape(&self) -> f64 {
-        self.shape
-    }
-
-    pub fn scale(&self) -> f64 {
-        self.scale
-    }
-}
-
 impl<Rhs, TRhs> Mul<Rhs> for Gamma
 where
-    Rhs: Distribution<T = TRhs, U = GammaParams>,
+    Rhs: Distribution<Value = TRhs, Condition = GammaParams>,
     TRhs: RandomVariable,
 {
     type Output = IndependentJoint<Self, Rhs, f64, TRhs, GammaParams>;
@@ -92,7 +64,7 @@ where
 
 impl<Rhs, URhs> BitAnd<Rhs> for Gamma
 where
-    Rhs: Distribution<T = GammaParams, U = URhs>,
+    Rhs: Distribution<Value = GammaParams, Condition = URhs>,
     URhs: RandomVariable,
 {
     type Output = DependentJoint<Self, Rhs, f64, GammaParams, URhs>;
