@@ -3,9 +3,11 @@ pub mod elliptical;
 pub mod exp;
 pub mod fisher_f;
 pub mod gamma;
+pub mod uniform;
 
 pub use beta::*;
 pub use elliptical::*;
 pub use exp::*;
 pub use fisher_f::*;
 pub use gamma::*;
+pub use uniform::*;
