@@ -91,9 +91,7 @@ where
 
 impl<L, R, T, UL, UR> ValueDifferentiableDistribution for DependentJoint<L, R, T, UL, UR>
 where
-    L: Distribution<Value = T, Condition = UL>
-        + ValueDifferentiableDistribution
-        + ConditionDifferentiableDistribution,
+    L: Distribution<Value = T, Condition = UL> + ValueDifferentiableDistribution,
     R: Distribution<Value = UL, Condition = UR> + ValueDifferentiableDistribution,
     T: RandomVariable,
     UL: RandomVariable,
@@ -131,8 +129,11 @@ where
 
 #[cfg(test)]
 mod tests {
-    use crate::distribution::Distribution;
     use crate::*;
+    use crate::{
+        ConditionDifferentiableDistribution, Distribution, ExactMultivariateNormalParams,
+        MultivariateNormal, ValueDifferentiableDistribution,
+    };
     use rand::prelude::*;
 
     #[test]
@@ -146,4 +147,28 @@ mod tests {
 
         println!("{:#?}", x);
     }
+
+    #[test]
+    fn it_works2() {
+        let model = Normal.condition(|x: &f64| NormalParams::new(1.0, x.powi(2) + 1.0)) & Normal;
+
+        let f = model
+            .ln_diff_value(&(1.0, 2.0), &NormalParams::new(0.0, 1.0).unwrap())
+            .unwrap();
+
+        println!("{:#?}", f);
+    }
+
+    #[test]
+    fn it_works3() {
+        let model_prior = Normal.condition(|x: &f64| NormalParams::new(1.0, x.powi(2) + 1.0));
+        let g = |theta: &f64| Ok(vec![0.0, 2.0 * theta]);
+        let model = ConditionDifferentiableConditionedDistribution::new(model_prior, g) & Normal;
+
+        let f = model
+            .ln_diff_condition(&(1.0, 2.0), &NormalParams::new(0.0, 1.0).unwrap())
+            .unwrap();
+
+        println!("{:#?}", f);
+    }
 }
