@@ -1,8 +1,9 @@
+pub mod axis;
 pub mod grid;
 pub mod regressor;
 
+pub use axis::*;
 pub use grid::*;
-pub use rayon::prelude::*;
 pub use regressor::*;
 
 use super::{BaseEllipticalProcessParams, EllipticalProcessParams};
@@ -11,7 +12,7 @@ use crate::{opensrdk_linear_algebra::*, RandomVariable};
 use crate::{DistributionError, EllipticalParams};
 use ey::y_ey;
 use opensrdk_kernel_method::*;
-use std::cmp::Ordering;
+use rayon::prelude::*;
 use std::error::Error;
 
 const K: usize = 100;
@@ -19,7 +20,7 @@ const K: usize = 100;
 #[derive(Clone, Debug)]
 pub struct KissLoveEllipticalProcessParams<K, T>
 where
-    K: Kernel<Vec<f64>>,
+    K: PositiveDefiniteKernel<Vec<f64>>,
     T: RandomVariable + Convolutable,
 {
     base: BaseEllipticalProcessParams<Convolutional<K>, T>,
@@ -30,13 +31,12 @@ where
     wx: Vec<SparseMatrix>,
     kuu: KroneckerMatrices,
     lkuu: KroneckerMatrices,
-    kxx_det_sqrt: f64,
     mahalanobis_squared: f64,
 }
 
 impl<K, T> KissLoveEllipticalProcessParams<K, T>
 where
-    K: Kernel<Vec<f64>>,
+    K: PositiveDefiniteKernel<Vec<f64>>,
     T: RandomVariable + Convolutable,
 {
     fn new(
@@ -65,7 +65,7 @@ where
             kuu.clone()
                 .eject()
                 .into_iter()
-                .map(|kpi| kpi.potrf())
+                .map(|kpi| -> Result<_, DistributionError> { Ok(kpi.potrf()?.0) })
                 .collect::<Result<Vec<_>, _>>()?,
         );
 
@@ -96,7 +96,7 @@ where
         let (q, t) = Matrix::sytrd_k(n, k, &|v| Self::sigma_mul(n, sigma2, &wx, &kuu, &v), None)?;
 
         // t = l * d * lt
-        let (l, d) = t.pttrf()?;
+        let pttrf_result = t.pttrf()?;
 
         let s = (0..p)
             .into_iter()
@@ -114,7 +114,7 @@ where
                     .map(|wx_q_col| Ok(kuu.vec_mul(wx_q_col.to_owned())?))
                     .collect::<Result<Vec<_>, DistributionError>>()?;
                 // r = qt * wxt * kuu
-                let r = Matrix::from(m, kuu_wx_r_cols.concat());
+                let r = Matrix::from(m, kuu_wx_r_cols.concat())?;
 
                 // kuu - rt * (l * d * lt)^{-1} * r = q2 * t2 * q2t
                 let (q2, t2) = Matrix::sytrd_k(
@@ -122,18 +122,18 @@ where
                     100,
                     &|v| {
                         Ok((kuu.vec_mul(v.clone())?.col_mat()
-                            - r.t() * l.pttrs(&d, &r * v.col_mat())?.vec().col_mat())
+                            - r.t() * pttrf_result.pttrs(&r * v.col_mat())?.vec().col_mat())
                         .vec())
                     },
                     None,
                 )?;
 
                 // t2 = l2 * d2 * l2t
-                let (l2, d2) = t2.pttrf()?;
-                let d2_sqrt = d2.dipowf(0.5);
+                let pttrf_result2 = t2.pttrf()?;
+                let d2_sqrt = pttrf_result2.1.powf(0.5);
                 // l2' = l2 * \sqrt{d2}
                 // kuu - rt * l * d * lt * r = q2 * l2' * l2't * q2t
-                let l2_prime = l2.mat(false) * d2_sqrt.mat();
+                let l2_prime = pttrf_result2.0.mat(false) * d2_sqrt.mat();
 
                 // s = q2 * l2'
                 let s = q2 * l2_prime;
@@ -142,8 +142,7 @@ where
             })
             .collect::<Result<Vec<_>, DistributionError>>()?;
 
-        let kxx_det_sqrt = Self::det_kxx(&wx, &kuu, sigma2)?.sqrt();
-        let mahalanobis_squared = (y_ey.t() * &sigma_inv_y)[0][0];
+        let mahalanobis_squared = (y_ey.t() * &sigma_inv_y)[(0, 0)];
 
         Ok(Self {
             base,
@@ -154,7 +153,6 @@ where
             wx,
             kuu,
             lkuu,
-            kxx_det_sqrt,
             mahalanobis_squared,
         })
     }
@@ -195,55 +193,6 @@ where
             })
     }
 
-    /// See Andrew Gordon Wilson
-    fn det_kxx(
-        wx: &Vec<SparseMatrix>,
-        kuu: &KroneckerMatrices,
-        sigma2: f64,
-    ) -> Result<f64, DistributionError> {
-        let m = wx[0].rows;
-        let n = wx[0].cols;
-
-        let kuu_toeplitz = kuu
-            .matrices()
-            .iter()
-            .map(|kp| Ok(ToeplitzMatrix::new(kp[0].to_vec(), kp[0][1..].to_vec())?))
-            .collect::<Result<Vec<_>, DistributionError>>()?;
-
-        let lambda = kuu_toeplitz
-            .par_iter()
-            .map(|kp| {
-                kp.embedded_circulant().cigv().1[..kp.dim()]
-                    .to_owned()
-                    .col_mat()
-            })
-            .collect::<Vec<_>>();
-        let lambda = KroneckerMatrices::new(lambda);
-        let mut lambda = lambda.prod().vec();
-
-        lambda.sort_by(|a, b| {
-            a.re.partial_cmp(&b.re).unwrap_or(if !a.re.is_finite() {
-                Ordering::Less
-            } else {
-                Ordering::Greater
-            })
-        });
-        if !lambda[0].re.is_finite() {
-            return Err(DistributionError::Others(
-                EllipticalProcessError::NaNContamination.into(),
-            ));
-        }
-
-        let lambda = &lambda[m - n..];
-
-        let det = lambda
-            .par_iter()
-            .map(|lmd| ((n as f64 / m as f64) * lmd.re + sigma2))
-            .product::<f64>();
-
-        Ok(det)
-    }
-
     fn sigma_mul(
         n: usize,
         sigma2: f64,
@@ -275,13 +224,13 @@ where
                 .concat(),
         );
 
-        Ok(sigma_inv_v)
+        Ok(sigma_inv_v?)
     }
 }
 
 impl<K, T> BaseEllipticalProcessParams<Convolutional<K>, T>
 where
-    K: Kernel<Vec<f64>>,
+    K: PositiveDefiniteKernel<Vec<f64>>,
     T: RandomVariable + Convolutable,
 {
     /// Lanczos Variance Estimate Kernel Interpolation for Scalable Structured Gaussian Process
@@ -298,9 +247,29 @@ where
     }
 }
 
+impl<K, T> RandomVariable for KissLoveEllipticalProcessParams<K, T>
+where
+    K: PositiveDefiniteKernel<Vec<f64>>,
+    T: RandomVariable + Convolutable,
+{
+    type RestoreInfo = ();
+
+    fn transform_vec(&self) -> (Vec<f64>, Self::RestoreInfo) {
+        todo!()
+    }
+
+    fn len(&self) -> usize {
+        todo!()
+    }
+
+    fn restore(v: &[f64], info: &Self::RestoreInfo) -> Result<Self, DistributionError> {
+        todo!()
+    }
+}
+
 impl<K, T> EllipticalParams for KissLoveEllipticalProcessParams<K, T>
 where
-    K: Kernel<Vec<f64>>,
+    K: PositiveDefiniteKernel<Vec<f64>>,
     T: RandomVariable + Convolutable,
 {
     fn mu(&self) -> &Vec<f64> {
@@ -314,10 +283,6 @@ where
         Self::sigma_inv_mul_with_params(n, sigma2, &self.wx, &self.kuu, &v)
     }
 
-    fn sigma_det_sqrt(&self) -> f64 {
-        self.kxx_det_sqrt
-    }
-
     fn lsigma_cols(&self) -> usize {
         self.mu.len()
     }
@@ -340,7 +305,7 @@ where
 
 impl<K, T> EllipticalProcessParams<Convolutional<K>, T> for KissLoveEllipticalProcessParams<K, T>
 where
-    K: Kernel<Vec<f64>>,
+    K: PositiveDefiniteKernel<Vec<f64>>,
     T: RandomVariable + Convolutable,
 {
     fn mahalanobis_squared(&self) -> f64 {
