@@ -9,23 +9,22 @@ use crate::{opensrdk_linear_algebra::*, RandomVariable};
 use crate::{DistributionError, EllipticalParams};
 use ey::y_ey;
 use opensrdk_kernel_method::*;
-
-const K: usize = 100;
+use opensrdk_linear_algebra::matrix::ge::sy_he::po::trf::POTRF;
 
 #[derive(Clone, Debug)]
 pub struct SparseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     base: BaseEllipticalProcessParams<K, T>,
     mu: Vec<f64>,
     lsigma: Matrix,
     u: Vec<T>,
-    lkuu: Matrix,
+    lkuu: POTRF,
     kux: Matrix,
     omega_inv: DiagonalMatrix,
-    ls: Matrix,
+    ls: POTRF,
     s_inv_kux_omega_y: Matrix,
     kxx_det_sqrt: f64,
     mahalanobis_squared: f64,
@@ -33,7 +32,7 @@ where
 
 impl<K, T> SparseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn new(
@@ -71,48 +70,43 @@ where
                 )
             })
             .map(|(kxixi, kuxi)| {
-                Ok(kxixi - (kuxi.t() * lkuu.potrs(kuxi)?)[0][0] + base.sigma.powi(2))
+                Ok(kxixi - (kuxi.t() * lkuu.potrs(kuxi)?)[(0, 0)] + base.sigma.powi(2))
             })
             .collect::<Result<Vec<_>, MatrixError>>()?
             .diag();
         let y_ey = y_ey(y, ey).col_mat();
-        let omega_y = &omega * y_ey.slice().to_vec();
+        let omega_y = &omega * y_ey.elems().to_vec();
 
-        let omega_inv = omega.diinv();
+        let omega_inv = omega.powi(-1);
         let omega_inv_mat = omega_inv.mat();
-        let s = &kuu + kux.t() * &omega_inv_mat * &kux;
+        let s = &kuu + &kux * &omega_inv_mat * kux.t();
         let ls = s.potrf()?;
 
-        let kux_ref = &kux;
         let omega_inv_ref = &omega_inv_mat;
-        let ls_ref = &ls;
-        let sigma_inv_mul = move |v: Vec<f64>| match Self::sigma_inv_mul(
-            kux_ref,
-            omega_inv_ref,
-            ls_ref,
-            v.col_mat(),
-        ) {
-            Ok(v) => Ok(v.vec()),
-            Err(e) => Err(e.into()),
-        };
+        // let sigma_inv_mul = move |v: Vec<f64>| match Self::sigma_inv_mul(
+        //     kux_ref,
+        //     omega_inv_ref,
+        //     ls_ref,
+        //     v.col_mat(),
+        // ) {
+        //     Ok(v) => Ok(v.vec()),
+        //     Err(e) => Err(e.into()),
+        // };
 
-        let k = n.min(K);
-        let (q, t) = Matrix::sytrd_k(n, k, &sigma_inv_mul, None)?;
-        let (bd, d) = t.pttrf()?;
-        let lkxx = kux.t() * (q * (bd.mat(false) * d.mat()));
+        let lsigma = Matrix::new(1, 1); // todo
 
         let s_inv_kux_omega_y = ls.potrs(&kux * omega_y.col_mat())?;
 
         let kxx_det_sqrt = 0.0; // todo
         let mahalanobis_squared =
-            (y_ey.t() * Self::sigma_inv_mul(&kux, omega_inv_ref, &ls, y_ey)?)[0][0];
+            (y_ey.t() * Self::sigma_inv_mul(&kux, omega_inv_ref, &ls, y_ey)?)[(0, 0)];
 
         base.x = vec![];
 
         Ok(Self {
             base,
             mu,
-            lsigma: lkxx,
+            lsigma,
             u,
             lkuu,
             kux,
@@ -127,7 +121,7 @@ where
     fn sigma_inv_mul(
         kux: &Matrix,
         omega_inv: &Matrix,
-        ls: &Matrix,
+        ls: &POTRF,
         v: Matrix,
     ) -> Result<Matrix, DistributionError> {
         Ok(omega_inv * v.clone() - omega_inv * kux.t() * ls.potrs(kux * omega_inv * v)?)
@@ -136,7 +130,7 @@ where
 
 impl<K, T> BaseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     /// Elliptical Process with approximation called the Fully Independent Training Conditional (FITC) for scalability.
@@ -153,9 +147,29 @@ where
     }
 }
 
+impl<K, T> RandomVariable for SparseEllipticalProcessParams<K, T>
+where
+    K: PositiveDefiniteKernel<T>,
+    T: RandomVariable,
+{
+    type RestoreInfo = ();
+
+    fn transform_vec(&self) -> (Vec<f64>, Self::RestoreInfo) {
+        todo!()
+    }
+
+    fn len(&self) -> usize {
+        todo!()
+    }
+
+    fn restore(v: &[f64], info: &Self::RestoreInfo) -> Result<Self, DistributionError> {
+        todo!()
+    }
+}
+
 impl<K, T> EllipticalParams for SparseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn mu(&self) -> &Vec<f64> {
@@ -166,10 +180,6 @@ where
         Self::sigma_inv_mul(&self.kux, &self.omega_inv.mat(), &self.ls, v)
     }
 
-    fn sigma_det_sqrt(&self) -> f64 {
-        self.kxx_det_sqrt
-    }
-
     fn lsigma_cols(&self) -> usize {
         self.lsigma.cols()
     }
@@ -181,7 +191,7 @@ where
 
 impl<K, T> EllipticalProcessParams<K, T> for SparseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn mahalanobis_squared(&self) -> f64 {
