@@ -5,11 +5,13 @@ use crate::{
     nonparametric::{kernel_matrix, regressor::GaussianProcessRegressor},
     ExactEllipticalParams, RandomVariable,
 };
-use opensrdk_kernel_method::Kernel;
+use opensrdk_kernel_method::PositiveDefiniteKernel;
+use opensrdk_linear_algebra::pp::trf::PPTRF;
+use opensrdk_linear_algebra::SymmetricPackedMatrix;
 
 impl<K, T> GaussianProcessRegressor<K, T> for SparseEllipticalProcessParams<K, T>
 where
-    K: Kernel<T>,
+    K: PositiveDefiniteKernel<T>,
     T: RandomVariable,
 {
     fn gp_predict_multivariate(
@@ -31,7 +33,8 @@ where
 
         let mean = self.mu[0] + &kxsu * &self.s_inv_kux_omega_y;
         let covariance = kxsxs - qxsxs + kxsu_s_inv_kuxs;
+        let cov_p = SymmetricPackedMatrix::from_mat(&covariance).unwrap();
 
-        ExactEllipticalParams::new(mean.vec(), covariance.potrf()?)
+        ExactEllipticalParams::new(mean.vec(), PPTRF(cov_p))
     }
 }
